<?php
/*
-------------------------------------------------------------
HessianPHP - Binary Web Services for PHP

Copyright (C) 2004-2005  by Manolo Gómez
http://www.hessianphp.org

Hessian Binary Web Service Protocol by Caucho(www.caucho.com) 

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

You can find the GNU General Public License here
http://www.gnu.org/licenses/lgpl.html
or in the license.txt file in your source directory.

If you have any questions or comments, please email:
vegeta.ec@gmail.com

*/

include_once 'Protocol.php';
include_once 'Hessian.php';
include_once 'Http.php';

/**
 * Represents a remote Hessian service endpoint with things such as
 * url, remote methods, security and several connection options
 * 
 * @package HessianPHP
 * @author Vegeta
 * @copyright Copyright (c) 2005
 * @version 1.0
 * @access public
 **/
class HessianProxy{
	var $url;
	var $remoteMethods = array();
	var $options;
	var $error = false;
	var $parser;
	var $writer;
	var $http;
	var $callingContext = array();

	function HessianProxy($url,$options=false){
		$this->url = $url;
		$this->writer = &new HessianWriter();
		$this->parser = &new HessianParser();
		$this->http = &new SocketHttpConnection($url,$options);
		$this->http->addHeader('Content-type','application/binary');
		$this->options = $options;
		$config = &HessianConfig::globalConfig();
		$this->errorLog = &$config->errorLog;

		// general options
		if(!empty($config->remoteMethods[$url])){
			foreach($config->remoteMethods[$url] as $method){
				$this->remoteMethod(trim($method));
			}
		}
		// local methods
		if(isset($options['methods'])){
			$methods = split(',',$options['methods']);
			foreach($methods as $method){
				$this->remoteMethod(trim($method));
			}
		}

	}

	/**
	 * Registers a remote method name. Useful for store description of services
	 * and resolve naming conflicts due to case sensitivity
	 *  
	 * @param string name Name of the remote method
	 * @access public
	 **/
	function remoteMethod($name){
		$phpmethod = strtolower($name);
		$this->remoteMethods[$phpmethod] = $name;
	}

	/**
	 * Returns the exact case sesitive name of a registered remote method
	 *  
	 * @param string method case insensitive name of the method
	 * @return string case sensitive name of the method
	 * @access public
	 **/
	function resolveMethod($method){
		$checkMethod = strtolower($method);
		// ugly
		$config = &HessianConfig::globalConfig();
		if(isset($config->remoteMethods[$this->url][$checkMethod])) {
			return $config->remoteMethods[$this->url][$checkMethod];
		}
		
		if(isset($this->remoteMethods[$checkMethod]))
			return $this->remoteMethods[$checkMethod];
		return $method;
	}
	
	/**
	 * Sets a connection option that will be passed to the Hessian proxy
	 * when called. Format is a pair key/value
	 *  
	 * @param string name Key
	 * @param string value Value
	 * @access public
	 **/
	function setOption($name,$value){
		$this->options[$name] = $value;
	}

	function getOption($name){
		if(isset($this->options[$name]))
			return $this->options[$name];
		return false;
	}	

	/**
	 * Notifies the HessianErrorLog object that handles the error
	 *  
	 * @param Object error An error object
	 * @return boolean always false, as it denotes an error
	 * @access public
	 **/

	function notifyError($error){
		$this->errorLog->notifyError($error);	
		return false;
	}

	function call($method,$params){
		if($this->http->hasError()) {
			return $this->notifyError($this->http->error);
		}

		$this->writer->clearRefs();

		$method = $this->resolveMethod($method);
		$data = &$this->writer->writeCall($method,$params);
		
		if(HessianError::isError($data)){
			return $this->notifyError($data);
		}

		$reply = $this->http->POST($data);
		if($this->http->hasError()) {
			return $this->notifyError($this->http->error);
		}

		$this->parser->setStream($reply);
		$this->parser->clearRefs();
		$result = &$this->parser->parseReply();
		if(HessianError::isError($result)){
			return $this->notifyError($result);
		}
		return $result;
	}
}

/**
 * <BLOCKQUOTE>
 * Proxy that allows RPC method calls to a remote Hessian compatible web-service
 * It requires the url of the Hessian service. Tested against Java and PHP services.
 * 
 * It uses the overload extension of PHP 4 to execute the remote calls. PHP 5 supports overloading
 * natively. If your PHP installation doesn't support overload, you can use direct calls to the __call() 
 * method using the arguments as detailed in <A HREF="http://us3.php.net/manual/en/ref.overload.php">http://us3.php.net/manual/en/ref.overload.php</A> 
 * or in the documentation of this class
 * <BR>
 * Sample usage:<BR>
 * <BR>
 * <code>
 * $testurl = 'http://www.caucho.com/hessian/test/basic';<BR>
 * $proxy = &new HessianClient($testurl);<BR>
 * echo $proxy->hello();<BR>
 * </code>
 * <BR>
 * <B>WARNING:</B> As in PHP 4.3.x, all method name metadata (get_class_methods(), overload extension __call()) is internally 
 * lower-cased so if you call a camel case remote method name, like 'getUsers', the call will result into a no such
 * method fault because PHP will interpret the method name as 'getusers'. This is especially bad when calling Java
 * services where many method names are camel cased. Sad but true.<BR>
 * <BR>
 * To counterfeit this, use {@link HessianPHP.Hessian#remoteMethod remoteMethod()}  in the proxy object to tell Hessian how to call the method with the 
 * right name before executing the call. PHP 5 doesn't seem to be affected by this behavior.
 * <BR>
 * <BR>TODO: 
 * <UL>
 *	<LI>add decent debug information</LI>
 *	<LI>SSL support</LI>
 *	<LI>gzip compression?</LI>
 *	<LI>Enhance error handling and test, test, test...</LI>
 * </UL>
 * </BLOCKQUOTE>
 * 
 * @package HessianPHP
 * @author Manolo Gómez
 * @copyright Copyright (c) 2004
 * @version 1.0
 * @access public
 **/

class HessianClient{
	/**
	 * Constructor, requires the url of the remote Hessian service
	 *  
	 * @param string url Url of the remote service
	 **/
	function HessianClient($url,$options=false){
		$this->__hessian__proxy__ = &Hessian::getHessianProxy($url,$options);
	}
	
	/**
	 * PHP magic function used to execute a remote call to a remote Hessian service.
	 *  
	 * @param string method Method name
	 * @param array params Arguments
	 * @param mixed return Returned value
	 * @return mixed True if PHP 4, return value of the function otherwise
 	 * @access public 
	 **/
	function __call($method,$params,&$return){
		$return = $this->__hessian__proxy__->call($method,$params);
		if (phpversion() < 5) return true;
		else return $return;
	}

}

// Call the overload() function when appropriate
if (function_exists('overload') && phpversion() < 5) {
   overload('HessianClient');
}



?>