<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2010 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik an der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki. If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

// This is not a valid entry point, perform no further processing unless MEDIAWIKI is defined
if( !defined( 'MEDIAWIKI' ) ) {
	echo "This file is part of Woogle4MediaWiki and is not a valid entry point\n";
	die( 1 );
}

/**
 * Main interface to implement by query backends
 * 
 * By default, this is implemented by WoogleNative (native PHP index)
 * and WoogleRemote (using TeamWeaverIS Java/Lucene-based backend)
 */
abstract class WoogleQueryService {

	// string written between different search result preview snippets
	public static $result_separator = ' ... ';
	// strings written around text to be highlighted in result snippets (e.g. text matching query) 
	public static $highlightStart = '<b>';
	public static $highlightStop = '</b>';
	
	
	/**
	 * checks if query connection works
	 * @return number of documents in index
	 */
	abstract public function checkQueryService();
	
	
	/**
	 * tells if this a native query service
	 * @return boolean
	 */
	abstract public function isNative();


	
	/**
	 * @return		array(string, ...)
	 */
	abstract public function getAttributeKeys();
	
	/**
	 * @param $key	string from getAttributeKeys()
	 * @return		array(
	 * 					array(
	 * 						'entity'	=> string,
	 * 						'name'		=> string,
	 * 						'freq'		=> int,
	 * 					),
	 * 					...
	 * 				)
	 */
	abstract public function getAttributeValues($key);
	
	/**
	 * @return		array(string, ...)
	 */
	abstract public function getSortKeys();
	
	/**
	 * Main method called by Woogle searches (via WoogleSearch)
	 * 
	 * @param $searchString		string. user search string
	 * @param $count			int, number of results to return
	 * @param $start			int, result number to start from (index in result list)
 	 * @param $advanced 		array of advanced search paramters
	 * @param $referer			string, denoting origin if query (e.g. 'mw' for queries coming from the built-in search form)
	 * @return 					array result list
	 */
	public function executeKeywordQuery($searchString, $count, $start, $advanced, $referer='') {
		WoogleUtil::profileIn(__METHOD__);
		
		$resultValues = array();
		
		// keys for advanced search array elements
		$rvTypes = array(
			'sources' => 'array', 'repositories' => 'array', 'mimetypes' => 'array',
			'changedMin' => 'date', 'changedMax' => 'date',
			'pagesMin' => 'int', 'pagesMax' => 'int', 'sortType' => 'int', 'excludeId' => 'int',
			'sortDesc' => 'bool',
			'sizeMin' => 'size', 'sizeMax' => 'size',
		);
		
		// iterate advanced search paramters
		foreach ($rvTypes as $rvKey => $rvValue) {
			switch ($rvValue) {
				case 'array':
					if (array_key_exists($rvKey, $advanced) && is_array($advanced[$rvKey])) {
						$resultValues[$rvKey] = $advanced[$rvKey];
					} else {
						$resultValues[$rvKey] = array();
					}
					break;
				case 'date':
					$resultValues[$rvKey] = 0;
					if (array_key_exists($rvKey, $advanced)) {
						$date = strtotime($advanced[$rvKey]);
						if ($date != false) {
							$resultValues[$rvKey] = $date;
						}
					}
					break;
				case 'int':
					$resultValues[$rvKey] = array_key_exists($rvKey, $advanced) ? intval($advanced[$rvKey]) : 0;
					break;
				case 'bool':
					if (!array_key_exists($rvKey, $advanced)) {
						$resultValues[$rvKey] = false;
					} elseif (is_bool($advanced[$rvKey])) {
						$resultValues[$rvKey] = $advanced[$rvKey];
					} else {
						$resultValues[$rvKey] = ($advanced[$rvKey] == 'true') ? true : false;
					}
					break;
				case 'size':
					if (array_key_exists($rvKey, $advanced) && array_key_exists('sizeFactor', $advanced)) {
						$resultValues[$rvKey] = intval($advanced[$rvKey] * pow(1024, $advanced['sizeFactor']));
					} else {
						$resultValues[$rvKey] = 0;
					}
					break;
			}
		}
		
		$resultList = $this->executeKeywordQueryValues($searchString, $count, $start, $resultValues, $referer);
		WoogleUtil::profileOut(__METHOD__);
		
//		WoogleUtil::info( 'Suchwort: '.$searchString);
//		WoogleUtil::info( print_r($resultList, true));
		
		return $resultList;
	}
	/**
	 * @param $searchString	string
	 * @param $count		int, how many results should be returned
	 * @param $start		int, where should the returned results start
	 * @param $values		array(
	 * 							string	=> int | string | bool | timestamp (int) | array,
	 * 							...
	 * 						)
	 * @param $referer		string, denoting origin if query (e.g. 'mw' for queries coming from the built-in search form)
	 * @return				array(
	 * 							'numHits'	=> int,
	 * 							'results'	=> array(
	 * 								array(
	 * 									'uri'		=> string,
	 * 									'label'		=> string,
	 * 									'content'	=> string,
	 * 									'mimetype'	=> string,
	 * 									'source'	=> string,
	 * 								),
	 * 								...
	 * 							),
	 * 						)
	 */
	abstract protected function executeKeywordQueryValues($searchString, $count, $start, $values, $referer='');
	
	/**
	 * @param $autocomplete	string, term for the autocompletion
	 * @param $num			int, how many results should be returned
	 * @return 				array(
	 * 							array(
	 * 								'entity'	=> string
	 * 								'name'		=> string
	 * 								'freq'		=> int
	 * 							)
	 * 						)
	 */
	abstract public function getAutocomplete($autocomplete, $num);
	
	/**
	 * This is the actual method which logs user result clicks. It is implemented by the
	 * respective implementation of WoogleQueryService. It is called by WoogleJobAddClick.
	 * 
	 * This method should not be called directly! Users should call the addClick method (see below)
	 * 
	 * @param $searchString		string, the query that produced the resultlist
	 * @param $url				string, result url
	 * @param $position			int, position in the resultlist
	 * @param $session			user session
	 * @return					void
	 */
	abstract public function doAddClick($searchString, $url, $position, $session);
	
	/**
	 * This method enqueues a user result click to the MediaWiki job queue. The job will
	 * later call doAddClick() to actually persist the log entry
	 * 
	 * @param $searchString		string, the query that produced the resultlist
	 * @param $url				string, result url
	 * @param $position			int, position in the resultlist
	 * @param $session			user session
	 * @return					void
	 */
	public function addClick($searchString, $url, $position, $session) {
		WoogleUtil::profileIn(__METHOD__);
		//WoogleJobAddClick::addJob($searchString, $url, $position, $session);
		$this->doAddClick($searchString, $url, $position, $session);
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * @return			bool, whether this QueryService supports requesting pages
	 */
	abstract public function supportPages();

	/**
	 * Surround text snippets strings with highlighting markup
	 * 
	 * @param $text			text to highlight
	 * @return				String, text, with surrounding highlight elements
	 */
	public static function highlight($text){
		return self::$highlightStart . $text . self::$highlightStop;
	}
	
	public static function getDummyValues(){
	  	return array(
		   	'mimetypes'  => null,
		   	'changedMin'  => 0,
		   	'changedMax'  => 0,
		   	'sizeMin'   => 0,
		   	'sizeMax'   => 0,
		   	'excludeId'  => 0,
		   	'sortType'   => 0,
	  	);
	}
}
